@@ -2,12 +2,10 @@
 This module provides an interface for invoking CMake executable.
 """
 
-from __future__ import print_function
 
 import argparse
-import distutils.sysconfig as du_sysconfig
+import contextlib
 import glob
-import io
 import itertools
 import os
 import os.path
@@ -17,6 +15,11 @@ import shlex
 import subprocess
 import sys
 import sysconfig
+from pathlib import Path
+from shlex import quote
+from typing import Dict, List, Mapping, Optional, Sequence, Tuple, overload
+
+import distutils.sysconfig as du_sysconfig
 
 from .constants import (
     CMAKE_BUILD_DIR,
@@ -27,22 +30,27 @@ from .constants import (
 from .exceptions import SKBuildError
 from .platform_specifics import get_platform
 
-if sys.version_info >= (3, 3):
-    from shlex import quote
-else:
-    from pipes import quote
-
 RE_FILE_INSTALL = re.compile(r"""[ \t]*file\(INSTALL DESTINATION "([^"]+)".*"([^"]+)"\).*""")
 
 
-def pop_arg(arg, args, default=None):
+@overload
+def pop_arg(arg: str, args: Sequence[str], default: None = None) -> Tuple[List[str], Optional[str]]:
+    ...
+
+
+@overload
+def pop_arg(arg: str, args: Sequence[str], default: str) -> Tuple[List[str], str]:
+    ...
+
+
+def pop_arg(arg: str, args: Sequence[str], default: Optional[str] = None) -> Tuple[List[str], Optional[str]]:
     """Pops an argument ``arg`` from an argument list ``args`` and returns the
     new list and the value of the argument if present and a default otherwise.
     """
     parser = argparse.ArgumentParser(add_help=False)
     parser.add_argument(arg)
-    namespace, args = parser.parse_known_args(args)
-    namespace = tuple(vars(namespace).items())
+    namespace_names, args = parser.parse_known_args(args)
+    namespace = tuple(vars(namespace_names).items())
     if namespace and namespace[0][1] is not None:
         val = namespace[0][1]
     else:
@@ -50,7 +58,7 @@ def pop_arg(arg, args, default=None):
     return args, val
 
 
-def _remove_cwd_prefix(path):
+def _remove_cwd_prefix(path: str) -> str:
     cwd = os.getcwd()
 
     result = path.replace("/", os.sep)
@@ -65,12 +73,12 @@ def _remove_cwd_prefix(path):
     return result
 
 
-def has_cmake_cache_arg(cmake_args, arg_name, arg_value=None):
+def has_cmake_cache_arg(cmake_args: List[str], arg_name: str, arg_value: Optional[str] = None) -> bool:
     """Return True if ``-D<arg_name>:TYPE=<arg_value>`` is found
     in ``cmake_args``. If ``arg_value`` is None, return True only if
     ``-D<arg_name>:`` is found in the list."""
     for arg in reversed(cmake_args):
-        if arg.startswith("-D{}:".format(arg_name)):
+        if arg.startswith(f"-D{arg_name}:"):
             if arg_value is None:
                 return True
             if "=" in arg:
@@ -78,7 +86,7 @@ def has_cmake_cache_arg(cmake_args, arg_name, arg_value=None):
     return False
 
 
-def get_cmake_version(cmake_executable=CMAKE_DEFAULT_EXECUTABLE):
+def get_cmake_version(cmake_executable: str = CMAKE_DEFAULT_EXECUTABLE) -> str:
     """
     Runs CMake and extracts associated version information.
     Raises :class:`skbuild.exceptions.SKBuildError` if it failed to execute CMake.
@@ -90,19 +98,18 @@ def get_cmake_version(cmake_executable=CMAKE_DEFAULT_EXECUTABLE):
         3.14.4
     """
     try:
-        version_string = subprocess.check_output([cmake_executable, "--version"])
-    except (OSError, subprocess.CalledProcessError):
+        version_string_bytes = subprocess.check_output([cmake_executable, "--version"])
+    except (OSError, subprocess.CalledProcessError) as err:
         raise SKBuildError(
-            "Problem with the CMake installation, aborting build. CMake executable is %s" % cmake_executable
-        )
+            f"Problem with the CMake installation, aborting build. CMake executable is {cmake_executable}"
+        ) from err
 
-    if sys.version_info > (3, 0):
-        version_string = version_string.decode()
+    version_string = version_string_bytes.decode()
 
     return version_string.splitlines()[0].split(" ")[-1]
 
 
-class CMaker(object):
+class CMaker:
     r"""Interface to CMake executable.
 
     Example:
@@ -136,33 +143,32 @@ class CMaker(object):
         >>>     cmkr.make(env=env)
     """
 
-    def __init__(self, cmake_executable=CMAKE_DEFAULT_EXECUTABLE):
+    def __init__(self, cmake_executable: str = CMAKE_DEFAULT_EXECUTABLE) -> None:
         self.cmake_executable = cmake_executable
         self.cmake_version = get_cmake_version(self.cmake_executable)
         self.platform = get_platform()
 
     @staticmethod
-    def get_cached(variable_name):
+    def get_cached(variable_name: str) -> Optional[str]:
         """If set, returns the variable cached value from the :func:`skbuild.constants.CMAKE_BUILD_DIR()`, otherwise returns None"""
-        variable_name = "{}:".format(variable_name)
-        try:
-            with open(os.path.join(CMAKE_BUILD_DIR(), "CMakeCache.txt")) as fp:
-                for line in fp:
-                    if line.startswith(variable_name):
-                        return line.split("=", 1)[-1].strip()
-        except (OSError, IOError):
-            pass
+        variable_name = f"{variable_name}:"
+        cmake_cache = Path(CMAKE_BUILD_DIR()) / "CMakeCache.txt"
+
+        with contextlib.suppress(OSError):
+            for line in cmake_cache.read_text("utf8").splitlines():
+                if line.startswith(variable_name):
+                    return line.split("=", 1)[-1].strip()
 
         return None
 
     @classmethod
-    def get_cached_generator_name(cls):
+    def get_cached_generator_name(cls) -> Optional[str]:
         """Reads and returns the cached generator from the :func:`skbuild.constants.CMAKE_BUILD_DIR()`:.
         Returns None if not found.
         """
         return cls.get_cached("CMAKE_GENERATOR")
 
-    def get_cached_generator_env(self):
+    def get_cached_generator_env(self) -> Optional[Dict[str, str]]:
         """If any, return a mapping of environment associated with the cached generator."""
         generator_name = self.get_cached_generator_name()
         if generator_name is not None:
@@ -172,14 +178,14 @@ class CMaker(object):
 
     def configure(
         self,
-        clargs=(),
-        generator_name=None,
-        skip_generator_test=False,
-        cmake_source_dir=".",
-        cmake_install_dir="",
-        languages=("C", "CXX"),
-        cleanup=True,
-    ):
+        clargs: Sequence[str] = (),
+        generator_name: Optional[str] = None,
+        skip_generator_test: bool = False,
+        cmake_source_dir: str = ".",
+        cmake_install_dir: str = "",
+        languages: Sequence[str] = ("C", "CXX"),
+        cleanup: bool = True,
+    ) -> Dict[str, str]:
         """Calls cmake to generate the Makefile/VS Solution/XCode project.
 
         clargs: tuple
@@ -241,12 +247,10 @@ class CMaker(object):
 
         ninja_executable_path = None
         if generator.name == "Ninja":
-            try:
+            with contextlib.suppress(ImportError):
                 import ninja  # pylint: disable=import-outside-toplevel
 
                 ninja_executable_path = os.path.join(ninja.BIN_DIR, "ninja")
-            except ImportError:
-                pass
 
         if not os.path.exists(CMAKE_BUILD_DIR()):
             os.makedirs(CMAKE_BUILD_DIR())
@@ -262,55 +266,56 @@ class CMaker(object):
         python_library = CMaker.get_python_library(python_version)
 
         cmake_source_dir = os.path.abspath(cmake_source_dir)
+        cmake_resource_dir = os.path.join(os.path.dirname(__file__), "resources", "cmake")
+        cmake_install_prefix = os.path.abspath(os.path.join(CMAKE_INSTALL_DIR(), cmake_install_dir))
+        python_version_string = sys.version.split(" ", maxsplit=1)[0]
 
         cmd = [
             self.cmake_executable,
             cmake_source_dir,
             "-G",
             generator.name,
-            ("-DCMAKE_INSTALL_PREFIX:PATH=" + os.path.abspath(os.path.join(CMAKE_INSTALL_DIR(), cmake_install_dir))),
-            ("-DPYTHON_VERSION_STRING:STRING=" + sys.version.split(" ")[0]),
-            ("-DSKBUILD:INTERNAL=" + "TRUE"),
-            ("-DCMAKE_MODULE_PATH:PATH=" + os.path.join(os.path.dirname(__file__), "resources", "cmake")),
+            f"-DCMAKE_INSTALL_PREFIX:PATH={cmake_install_prefix}",
+            f"-DPYTHON_VERSION_STRING:STRING={python_version_string}",
+            "-DSKBUILD:INTERNAL=TRUE",
+            f"-DCMAKE_MODULE_PATH:PATH={cmake_resource_dir}",
+            f"-DPYTHON_EXECUTABLE:PATH={sys.executable}",
+            f"-DPYTHON_INCLUDE_DIR:PATH={python_include_dir}",
+            f"-DPYTHON_LIBRARY:PATH={python_library}",
         ]
 
-        find_python_prefixes = [
-            "-DPython{}".format(python_version[0]),
-            "-DPython",
-            "-DPYTHON",
-        ]
-
-        for prefix in find_python_prefixes:
+        for prefix in ["-DPython", "-DPython3"]:
             cmd.extend(
                 [
-                    (prefix + "_EXECUTABLE:FILEPATH=" + sys.executable),
-                    (prefix + "_INCLUDE_DIR:PATH=" + python_include_dir),
-                    (prefix + "_LIBRARY:PATH=" + python_library),
+                    f"{prefix}_EXECUTABLE:PATH={sys.executable}",
+                    f"{prefix}_ROOT_DIR:PATH={sys.prefix}",
+                    f"{prefix}_INCLUDE_DIR:PATH={python_include_dir}",
+                    f"{prefix}_FIND_REGISTRY:STRING=NEVER",
                 ]
             )
+            if sys.implementation.name == "pypy":
+                cmd.append(f"{prefix}_FIND_IMPLEMENTATIONS:STRING=PyPy")
 
-            try:
+            with contextlib.suppress(ImportError):
                 import numpy as np
 
-                cmd.append(prefix + "_NumPy_INCLUDE_DIRS:PATH=" + np.get_include())
-            except ImportError:
-                pass
+                cmd.append(f"{prefix}_NumPy_INCLUDE_DIRS:PATH=" + np.get_include())
 
         if generator.toolset:
             cmd.extend(["-T", generator.toolset])
         if generator.architecture:
             cmd.extend(["-A", generator.architecture])
         if ninja_executable_path is not None:
-            cmd.append("-DCMAKE_MAKE_PROGRAM:FILEPATH=" + ninja_executable_path)
+            cmd.append(f"-DCMAKE_MAKE_PROGRAM:FILEPATH={ninja_executable_path}")
 
         cmd.extend(clargs)
 
         # Parse CMAKE_ARGS only if SKBUILD_CONFIGURE_OPTIONS is not present
         if "SKBUILD_CONFIGURE_OPTIONS" in os.environ:
-            env_cmake_args = filter(None, shlex.split(os.environ["SKBUILD_CONFIGURE_OPTIONS"]))
+            env_cmake_args = list(filter(None, shlex.split(os.environ["SKBUILD_CONFIGURE_OPTIONS"])))
         else:
-            env_cmake_args = filter(None, shlex.split(os.environ.get("CMAKE_ARGS", "")))
-            env_cmake_args = [s for s in env_cmake_args if "CMAKE_INSTALL_PREFIX" not in s]
+            env_cmake_args_filtered = filter(None, shlex.split(os.environ.get("CMAKE_ARGS", "")))
+            env_cmake_args = [s for s in env_cmake_args_filtered if "CMAKE_INSTALL_PREFIX" not in s]
 
         cmd.extend(env_cmake_args)
 
@@ -319,25 +324,22 @@ class CMaker(object):
         print(
             "Configuring Project\n"
             "  Working directory:\n"
-            "    {}\n"
+            f"    {os.path.abspath(CMAKE_BUILD_DIR())}\n"
             "  Command:\n"
-            "    {}\n".format(os.path.abspath(CMAKE_BUILD_DIR()), self._formatArgsForDisplay(cmd))
+            f"    {self._formatArgsForDisplay(cmd)}\n",
+            flush=True,
         )
         rtn = subprocess.call(cmd, cwd=CMAKE_BUILD_DIR(), env=generator.env)
         if rtn != 0:
             raise SKBuildError(
                 "An error occurred while configuring with CMake.\n"
                 "  Command:\n"
-                "    {}\n"
+                f"    {self._formatArgsForDisplay(cmd)}\n"
                 "  Source directory:\n"
-                "    {}\n"
+                f"    {os.path.abspath(cmake_source_dir)}\n"
                 "  Working directory:\n"
-                "    {}\n"
-                "Please see CMake's output for more information.".format(
-                    self._formatArgsForDisplay(cmd),
-                    os.path.abspath(cmake_source_dir),
-                    os.path.abspath(CMAKE_BUILD_DIR()),
-                )
+                f"    {os.path.abspath(CMAKE_BUILD_DIR())}\n"
+                "Please see CMake's output for more information."
             )
 
         CMaker.check_for_bad_installs()
@@ -345,7 +347,7 @@ class CMaker(object):
         return generator.env
 
     @staticmethod
-    def get_python_version():
+    def get_python_version() -> str:
         """Get version associated with the current python interpreter.
 
         Returns:
@@ -366,12 +368,14 @@ class CMaker(object):
         if not python_version:
             python_version = ".".join(map(str, sys.version_info[:2]))
 
+        assert isinstance(python_version, str)
+
         return python_version
 
     # NOTE(opadron): The try-excepts raise the cyclomatic complexity, but we
     # need them for this function.
     @staticmethod  # noqa: C901
-    def get_python_include_dir(python_version):
+    def get_python_include_dir(python_version: str) -> Optional[str]:
         """Get include directory associated with the current python
         interpreter.
 
@@ -390,7 +394,7 @@ class CMaker(object):
             python_include_dir = '.../conda/envs/py37/include/python3.7m'
         """
         # determine python include dir
-        python_include_dir = sysconfig.get_config_var("INCLUDEPY")
+        python_include_dir: Optional[str] = sysconfig.get_config_var("INCLUDEPY")
 
         # if Python.h not found (or python_include_dir is None), try to find a
         # suitable include dir
@@ -401,27 +405,21 @@ class CMaker(object):
             # NOTE(opadron): these possible prefixes must be guarded against
             # AttributeErrors and KeyErrors because they each can throw on
             # different platforms or even different builds on the same platform.
-            include_py = sysconfig.get_config_var("INCLUDEPY")
-            include_dir = sysconfig.get_config_var("INCLUDEDIR")
-            include = None
-            plat_include = None
-            python_inc = None
-            python_inc2 = None
+            include_py: Optional[str] = sysconfig.get_config_var("INCLUDEPY")
+            include_dir: Optional[str] = sysconfig.get_config_var("INCLUDEDIR")
+            include: Optional[str] = None
+            plat_include: Optional[str] = None
+            python_inc: Optional[str] = None
+            python_inc2: Optional[str] = None
 
-            try:
+            with contextlib.suppress(AttributeError, KeyError):
                 include = sysconfig.get_path("include")
-            except (AttributeError, KeyError):
-                pass
 
-            try:
+            with contextlib.suppress(AttributeError, KeyError):
                 plat_include = sysconfig.get_path("platinclude")
-            except (AttributeError, KeyError):
-                pass
 
-            try:
-                python_inc = sysconfig.get_python_inc()
-            except AttributeError:
-                pass
+            with contextlib.suppress(AttributeError):
+                python_inc = sysconfig.get_python_inc()  # type: ignore[attr-defined]
 
             if include_py is not None:
                 include_py = os.path.dirname(include_py)
@@ -432,29 +430,16 @@ class CMaker(object):
             if python_inc is not None:
                 python_inc2 = os.path.join(python_inc, ".".join(map(str, sys.version_info[:2])))
 
-            candidate_prefixes = list(
-                filter(
-                    bool,
-                    (
-                        include_py,
-                        include_dir,
-                        include,
-                        plat_include,
-                        python_inc,
-                        python_inc2,
-                    ),
-                )
-            )
+            all_candidate_prefixes = [include_py, include_dir, include, plat_include, python_inc, python_inc2]
+            candidate_prefixes: List[str] = [pre for pre in all_candidate_prefixes if pre]
 
-            candidate_versions = (python_version,)
+            candidate_versions: Tuple[str, ...] = (python_version,)
             if python_version:
                 candidate_versions += ("",)
 
                 pymalloc = None
-                try:
+                with contextlib.suppress(AttributeError):
                     pymalloc = bool(sysconfig.get_config_var("WITH_PYMALLOC"))
-                except AttributeError:
-                    pass
 
                 if pymalloc:
                     candidate_versions += (python_version + "m",)
@@ -476,7 +461,7 @@ class CMaker(object):
         return python_include_dir
 
     @staticmethod
-    def get_python_library(python_version):
+    def get_python_library(python_version: str) -> Optional[str]:
         """Get path to the python library associated with the current python
         interpreter.
 
@@ -498,8 +483,8 @@ class CMaker(object):
         # modern python versions that avoids the complicated construct below.
         # It avoids guessing the library name. Tested with cpython 3.8 and
         # pypy 3.8 on Ubuntu.
-        libdir = sysconfig.get_config_var("LIBDIR")
-        ldlibrary = sysconfig.get_config_var("LDLIBRARY")
+        libdir: Optional[str] = sysconfig.get_config_var("LIBDIR")
+        ldlibrary: Optional[str] = sysconfig.get_config_var("LDLIBRARY")
         if libdir and ldlibrary and os.path.exists(libdir):
             if sysconfig.get_config_var("MULTIARCH"):
                 masd = sysconfig.get_config_var("multiarchsubdir")
@@ -510,15 +495,15 @@ class CMaker(object):
                     if os.path.exists(libdir_masd):
                         libdir = libdir_masd
             libpath = os.path.join(libdir, ldlibrary)
-            if os.path.exists(libpath):
+            if libpath and os.path.exists(libpath):
                 return libpath
 
         return CMaker._guess_python_library(python_version)
 
     @staticmethod
-    def _guess_python_library(python_version):
+    def _guess_python_library(python_version: str) -> Optional[str]:
         # determine direct path to libpython
-        python_library = sysconfig.get_config_var("LIBRARY")
+        python_library: Optional[str] = sysconfig.get_config_var("LIBRARY")
 
         # if static (or nonexistent), try to find a suitable dynamic libpython
         if not python_library or os.path.splitext(python_library)[1][-2:] == ".a":
@@ -558,10 +543,10 @@ class CMaker(object):
             # get the value of `LIBDIR`.
             candidate_libdirs = []
             libdir_a = du_sysconfig.get_config_var("LIBDIR")
+            assert not isinstance(libdir_a, int)
             if libdir_a is None:
-                candidate_libdirs.append(
-                    os.path.abspath(os.path.join(sysconfig.get_config_var("LIBDEST"), "..", "libs"))
-                )
+                libdest = sysconfig.get_config_var("LIBDEST")
+                candidate_libdirs.append(os.path.abspath(os.path.join(libdest, "..", "libs") if libdest else "libs"))
             libdir_b = sysconfig.get_config_var("LIBDIR")
             for libdir in (libdir_a, libdir_b):
                 if libdir is None:
@@ -598,7 +583,7 @@ class CMaker(object):
         return python_library
 
     @staticmethod
-    def check_for_bad_installs():
+    def check_for_bad_installs() -> None:
         """This function tries to catch files that are meant to be installed
         outside the project root before they are actually installed.
 
@@ -618,7 +603,7 @@ class CMaker(object):
                 if os.path.splitext(filename)[1] != ".cmake":
                     continue
 
-                with io.open(os.path.join(root, filename), encoding="utf-8") as fp:
+                with open(os.path.join(root, filename), encoding="utf-8") as fp:
                     lines = fp.readlines()
 
                 for line in lines:
@@ -644,7 +629,14 @@ class CMaker(object):
                 )
             )
 
-    def make(self, clargs=(), config="Release", source_dir=".", install_target="install", env=None):
+    def make(
+        self,
+        clargs: Sequence[str] = (),
+        config: str = "Release",
+        source_dir: str = ".",
+        install_target: str = "install",
+        env: Optional[Mapping[str, str]] = None,
+    ) -> None:
         """Calls the system-specific make program to compile code.
 
         install_target: string
@@ -663,9 +655,9 @@ class CMaker(object):
         clargs, install_target = pop_arg("--install-target", clargs, install_target)
         if not os.path.exists(CMAKE_BUILD_DIR()):
             raise SKBuildError(
-                ("CMake build folder ({}) does not exist. " "Did you forget to run configure before " "make?").format(
-                    CMAKE_BUILD_DIR()
-                )
+                f"CMake build folder ({CMAKE_BUILD_DIR()}) does not exist. "
+                "Did you forget to run configure before "
+                "make?"
             )
 
         # Workaround CMake issue #8438
@@ -679,7 +671,14 @@ class CMaker(object):
 
         self.make_impl(clargs=clargs, config=config, source_dir=source_dir, install_target=install_target, env=env)
 
-    def make_impl(self, clargs, config, source_dir, install_target, env=None):
+    def make_impl(
+        self,
+        clargs: List[str],
+        config: str,
+        source_dir: str,
+        install_target: Optional[str],
+        env: Optional[Mapping[str, str]] = None,
+    ) -> None:
         """
         Precondition: clargs does not have --config nor --install-target options.
         These command line arguments are extracted in the caller function
@@ -704,29 +703,24 @@ class CMaker(object):
             raise SKBuildError(
                 "An error occurred while building with CMake.\n"
                 "  Command:\n"
-                "    {}\n"
+                f"    {self._formatArgsForDisplay(cmd)}\n"
                 "  Install target:\n"
-                "    {}\n"
+                f"    {install_target}\n"
                 "  Source directory:\n"
-                "    {}\n"
+                f"    {os.path.abspath(source_dir)}\n"
                 "  Working directory:\n"
-                "    {}\n"
+                f"    {os.path.abspath(CMAKE_BUILD_DIR())}\n"
                 "Please check the install target is valid and see CMake's output for more "
-                "information.".format(
-                    self._formatArgsForDisplay(cmd),
-                    install_target,
-                    os.path.abspath(source_dir),
-                    os.path.abspath(CMAKE_BUILD_DIR()),
-                )
+                "information."
             )
 
-    def install(self):
+    def install(self) -> List[str]:
         """Returns a list of file paths to install via setuptools that is
         compatible with the data_files keyword argument.
         """
         return self._parse_manifests()
 
-    def _parse_manifests(self):
+    def _parse_manifests(self) -> List[str]:
         paths = glob.glob(os.path.join(CMAKE_BUILD_DIR(), "install_manifest*.txt"))
         try:
             return [self._parse_manifest(path) for path in paths][0]
@@ -734,14 +728,12 @@ class CMaker(object):
             return []
 
     @staticmethod
-    def _parse_manifest(install_manifest_path):
-        with open(install_manifest_path, "r") as manifest:
+    def _parse_manifest(install_manifest_path: str) -> List[str]:
+        with open(install_manifest_path, encoding="utf-8") as manifest:
             return [_remove_cwd_prefix(path) for path in manifest]
 
-        return []
-
     @staticmethod
-    def _formatArgsForDisplay(args):
+    def _formatArgsForDisplay(args: Sequence[str]) -> str:
         """Format a list of arguments appropriately for display. When formatting
         a command and its arguments, the user should be able to execute the
         command by copying and pasting the output directly into a shell.
